import re
from io import BytesIO

from webdav3.client import Client as WebDAVclient

from .settings import SETTINGS

_WEBDAV_CLIENT = WebDAVclient({
    "webdav_hostname":  SETTINGS.webdav.url,
    "webdav_login":     SETTINGS.webdav.username,
    "webdav_password":  SETTINGS.webdav.password,
    "disable_check":    SETTINGS.webdav.disable_check,
})


async def list_files(regex: re.Pattern, path: str = "") -> list[str]:
    ls = _WEBDAV_CLIENT.list(path)
    return [
        path
        for path in ls
        if regex.search(path)
    ]


async def get_file(path: str) -> BytesIO:
    resource = _WEBDAV_CLIENT.resource(path)
    buffer = BytesIO()
    resource.write_to(buffer)
    buffer.seek(0)

    return buffer
